/**
 ******************************************************************************
 * @file    sht31dis.c
 * @author  ELKYS servis s.r.o. - Radek Kysilka 2019
 * @version V1.0.0
 * @date    14-February-2019
 * @brief   This file provides a set of functions needed to manage the SHT31DIS
 *          humidity and temperature devices
 ******************************************************************************
 * @attention
 *
 * <h2><center>&copy; COPYRIGHT(c) 2017 ELKYS servis s.r.o.</center></h2>
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *   1. Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *   3. Neither the name of ELKYS servis s.r.o. nor the names of its contributors
 *      may be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "sht31dis.h"


/* SHT31DIS Humidity Private Variables */
HSENSOR_DrvTypeDef SHT31DIS_H_Drv =
{
  SHT31DIS_H_Init,
  SHT31DIS_H_ReadID,
  SHT31DIS_H_ReadHumidity
};

/* SHT31DIS_Temperature_Private_Variables */
TSENSOR_DrvTypeDef SHT31DIS_T_Drv =
{
  SHT31DIS_T_Init,
  0,
  0,
  SHT31DIS_T_ReadTemp
};

float SHT31DIS_LastValue[2] = {0, 0};

unsigned char SHT31DIS_WriteCommand(uint16_t DeviceAddr, SHT31DIS_Commands cmd)
{
	SENSOR_IO_Write(DeviceAddr, (uint8_t)((cmd>>8)&0xff), (uint8_t)(cmd&0xff));
	return 1;
}
/*
*
* CRC-8 formula from page 14 of SHT spec pdf
*
* Test data 0xBE, 0xEF should yield 0x92
*
* Initialization data 0xFF
* Polynomial 0x31 (x8 + x5 +x4 +1)
* Final XOR 0x00
*/
uint8_t SHT31DIS_crc8(const uint8_t *data, int len)
{
  const uint8_t POLYNOMIAL = 0x31;
  uint8_t crc = 0xFF;
  int j;
  int i;

  for (j = len; j; --j ) {
    crc ^= *data++;

    for ( i = 8; i; --i ) {
      crc = ( crc & 0x80 )
            ? (crc << 1) ^ POLYNOMIAL
            : (crc << 1);
    }
  }
  return crc;
}

/**
  * @brief  Set SHT31DIS humidity sensor Initialization.
  */
void SHT31DIS_H_Init(uint16_t DeviceAddr)
{
  
	/* Polling device */
	SHT31DIS_WriteCommand(DeviceAddr, CMD_MEAS_POLLING_H);
}

/**
  * @brief  Read SHT31DIS ID.
  * @retval ID 
  */
uint8_t SHT31DIS_H_ReadID(uint16_t DeviceAddr)
{  
  return SHT31DIS_WHO_AM_I_VAL;
}

void SHT31DIS_H_ReadData(uint16_t DeviceAddr)
{
	HAL_StatusTypeDef status;
	uint8_t data[6];

	status = SENSOR_IO_Receive(DeviceAddr, data, 6);
	SHT31DIS_WriteCommand(DeviceAddr, CMD_MEAS_POLLING_H);
	if (status != HAL_OK) return;

	//Check crc for humidity
	if (data[5] != SHT31DIS_crc8(&data[3], 2)) return;
	SHT31DIS_LastValue[0] = 100*(float)((data[3] << 8) | data[4])/65535;

	if (data[2] != SHT31DIS_crc8(&data[0], 2)) return;
	SHT31DIS_LastValue[1] = (175*(float)((data[0] << 8) | data[1])/65535) - 45;
	return;
}

/**
  * @brief  Read humidity value of SHT31DIS
  * @retval humidity value;
  */
float SHT31DIS_H_ReadHumidity(uint16_t DeviceAddr)
{
	SHT31DIS_H_ReadData(DeviceAddr);
	return SHT31DIS_LastValue[0];
}


/**
  * @}
  */ 

/** @defgroup SHT31DIS_Temperature_Private_Functions SHT31DIS Temperature Private Functions
  * @{
  */

/**
  * @brief  Set SHT31DIS temperature sensor Initialization.
  * @param  DeviceAddr: I2C device address
  * @param  InitStruct: pointer to a TSENSOR_InitTypeDef structure 
  *         that contains the configuration setting for the SHT31DIS.
  */
void SHT31DIS_T_Init(uint16_t DeviceAddr, TSENSOR_InitTypeDef *pInitStruct)
{  
	/* Polling device */
	SHT31DIS_WriteCommand(DeviceAddr, CMD_MEAS_POLLING_H);
}

/**
  * @brief  Read temperature value of SHT31DIS
  * @param  DeviceAddr: I2C device address
  * @retval temperature value
  */
float SHT31DIS_T_ReadTemp(uint16_t DeviceAddr)
{
	SHT31DIS_H_ReadData(DeviceAddr);
	return SHT31DIS_LastValue[1];
}

