/* Includes ------------------------------------------------------------------*/
#include "board.h"


/**
 * @brief BOARD BSP Driver version number
   */
#define __BOARD_BSP_VERSION_MAIN   (0x01) /*!< [31:24] main version */
#define __BOARD_BSP_VERSION_SUB1   (0x01) /*!< [23:16] sub1 version */
#define __BOARD_BSP_VERSION_SUB2   (0x00) /*!< [15:8]  sub2 version */
#define __BOARD_BSP_VERSION_RC     (0x00) /*!< [7:0]  release candidate */
#define __BOARD_BSP_VERSION            ((__BOARD_BSP_VERSION_MAIN << 24)\
									  |(__BOARD_BSP_VERSION_SUB1 << 16)\
									  |(__BOARD_BSP_VERSION_SUB2 << 8 )\
									  |(__BOARD_BSP_VERSION_RC))
/**
  * @}
  */

/*
 * I2C routine header
 */
static HAL_StatusTypeDef I2Cx_ReadMultiple(I2C_HandleTypeDef *i2c_handler, uint8_t Addr, uint16_t Reg, uint16_t MemAddSize, uint8_t *Buffer, uint16_t Length);
static HAL_StatusTypeDef I2Cx_WriteMultiple(I2C_HandleTypeDef *i2c_handler, uint8_t Addr, uint16_t Reg, uint16_t MemAddSize, uint8_t *Buffer, uint16_t Length);
static HAL_StatusTypeDef I2Cx_IsDeviceReady(I2C_HandleTypeDef *i2c_handler, uint16_t DevAddress, uint32_t Trials);
static void              I2Cx_Error(I2C_HandleTypeDef *i2c_handler, uint8_t Addr);


/* Sensors IO functions */
void     SENSOR_IO_Init(void);
void     SENSOR_IO_DeInit(void);
void     SENSOR_IO_Write(uint8_t Addr, uint8_t Reg, uint8_t Value);
uint8_t  SENSOR_IO_Read(uint8_t Addr, uint8_t Reg);
uint16_t SENSOR_IO_ReadMultiple(uint8_t Addr, uint8_t Reg, uint8_t *Buffer, uint16_t Length);
void     SENSOR_IO_WriteMultiple(uint8_t Addr, uint8_t Reg, uint8_t *Buffer, uint16_t Length);
HAL_StatusTypeDef SENSOR_IO_Receive(uint8_t Addr, uint8_t *Buffer, uint16_t Length);
HAL_StatusTypeDef SENSOR_IO_IsDeviceReady(uint16_t DevAddress, uint32_t Trials);
void     SENSOR_IO_Delay(uint32_t Delay);

/* DSRC IO functions */
/*void DSRC_IO_Init(void);
void DRSC_IO_DeInit(void);
void DRSC_IO_Write(uint8_t *pBuffer, uint16_t NumByteToWrite);
uint16_t DRSC_IO_Read(uint8_t *pBuffer);*/

/* Charger IO functions */
void     CHARGER_IO_Init(void);
void     CHARGER_IO_DeInit(void);
void     CHARGER_IO_Write(uint8_t Addr, uint8_t Reg, uint8_t Value);
uint8_t  CHARGER_IO_Read(uint8_t Addr, uint8_t Reg);
uint16_t CHARGER_IO_ReadMultiple(uint8_t Addr, uint8_t Reg, uint8_t *Buffer, uint16_t Length);
void     CHARGER_IO_WriteMultiple(uint8_t Addr, uint8_t Reg, uint8_t *Buffer, uint16_t Length);
HAL_StatusTypeDef CHARGER_IO_IsDeviceReady(uint16_t DevAddress, uint32_t Trials);
void     CHARGER_IO_Delay(uint32_t Delay);

/* GNSS IO functions */
void GNSS_IO_Power(FunctionalState on);



uint32_t BSP_GetVersion(void)
{
  return __BOARD_BSP_VERSION;
}



/*******************************************************************************
                            BUS OPERATIONS
*******************************************************************************/
/******************************* I2C Routines *********************************/


/**
  * @brief  Reads multiple data.
  * @param  i2c_handler : I2C handler
  * @param  Addr: I2C address
  * @param  Reg: Reg address
  * @param  MemAddress: memory address
  * @param  Buffer: Pointer to data buffer
  * @param  Length: Length of the data
  * @retval HAL status
  */
static HAL_StatusTypeDef I2Cx_ReadMultiple(I2C_HandleTypeDef *i2c_handler, uint8_t Addr, uint16_t Reg, uint16_t MemAddress, uint8_t *Buffer, uint16_t Length)
{
  HAL_StatusTypeDef status = HAL_OK;

  status = HAL_I2C_Mem_Read(i2c_handler, Addr, (uint16_t)Reg, MemAddress, Buffer, Length, 1000);

  /* Check the communication status */
  if(status != HAL_OK)
  {
    /* I2C error occured */
    I2Cx_Error(i2c_handler, Addr);
  }
  return status;
}

/**
  * @brief  Writes a value in a register of the device through BUS in using DMA mode.
  * @param  i2c_handler : I2C handler
  * @param  Addr: Device address on BUS Bus.
  * @param  Reg: The target register address to write
  * @param  MemAddress: memory address
  * @param  Buffer: The target register value to be written
  * @param  Length: buffer size to be written
  * @retval HAL status
  */
static HAL_StatusTypeDef I2Cx_WriteMultiple(I2C_HandleTypeDef *i2c_handler, uint8_t Addr, uint16_t Reg, uint16_t MemAddress, uint8_t *Buffer, uint16_t Length)
{
  HAL_StatusTypeDef status = HAL_OK;

  status = HAL_I2C_Mem_Write(i2c_handler, Addr, (uint16_t)Reg, MemAddress, Buffer, Length, 1000);

  /* Check the communication status */
  if(status != HAL_OK)
  {
    /* Re-Initiaize the I2C Bus */
    I2Cx_Error(i2c_handler, Addr);
  }
  return status;
}

/**
  * @brief  Checks if target device is ready for communication.
  * @note   This function is used with Memory devices
  * @param  i2c_handler : I2C handler
  * @param  DevAddress: Target device address
  * @param  Trials: Number of trials
  * @retval HAL status
  */
static HAL_StatusTypeDef I2Cx_IsDeviceReady(I2C_HandleTypeDef *i2c_handler, uint16_t DevAddress, uint32_t Trials)
{
  return (HAL_I2C_IsDeviceReady(i2c_handler, DevAddress, Trials, 1000));
}

/**
  * @brief  Manages error callback by re-initializing I2C.
  * @param  i2c_handler : I2C handler
  * @param  Addr: I2C Address
  * @retval None
  */
static void I2Cx_Error(I2C_HandleTypeDef *i2c_handler, uint8_t Addr)
{
  /* De-initialize the I2C communication bus */
  //HAL_I2C_DeInit(i2c_handler);

  /* Re-Initialize the I2C communication bus */
  //I2Cx_Init(i2c_handler);
}



/**
  * @brief  Initializes Sensors low level.
  * @retval None
  */
void SENSOR_IO_Init(void)
{
  //I2Cx_Init(&hI2cHandler);
}

/**
  * @brief  DeInitializes Sensors low level.
  * @retval None
  */
void SENSOR_IO_DeInit(void)
{
  //I2Cx_DeInit(&hI2cHandler);
}

/**
  * @brief  Writes a single data.
  * @param  Addr: I2C address
  * @param  Reg: Reg address
  * @param  Value: Data to be written
  * @retval None
  */
void SENSOR_IO_Write(uint8_t Addr, uint8_t Reg, uint8_t Value)
{
  I2Cx_WriteMultiple(&SENSOR_I2C_BUS_HANDLER, Addr, (uint16_t)Reg, I2C_MEMADD_SIZE_8BIT,(uint8_t*)&Value, 1);
}

/**
  * @brief  Reads a single data.
  * @param  Addr: I2C address
  * @param  Reg: Reg address
  * @retval Data to be read
  */
uint8_t SENSOR_IO_Read(uint8_t Addr, uint8_t Reg)
{
  uint8_t read_value = 0;

  I2Cx_ReadMultiple(&SENSOR_I2C_BUS_HANDLER, Addr, Reg, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&read_value, 1);

  return read_value;
}

/**
  * @brief  Reads multiple data with I2C communication
  *         channel from TouchScreen.
  * @param  Addr: I2C address
  * @param  Reg: Register address
  * @param  Buffer: Pointer to data buffer
  * @param  Length: Length of the data
  * @retval HAL status
  */
uint16_t SENSOR_IO_ReadMultiple(uint8_t Addr, uint8_t Reg, uint8_t *Buffer, uint16_t Length)
{
 return I2Cx_ReadMultiple(&SENSOR_I2C_BUS_HANDLER, Addr, (uint16_t)Reg, I2C_MEMADD_SIZE_8BIT, Buffer, Length);
}

HAL_StatusTypeDef SENSOR_IO_Receive(uint8_t Addr, uint8_t *Buffer, uint16_t Length)
{
	return HAL_I2C_Master_Receive(&SENSOR_I2C_BUS_HANDLER, Addr, Buffer, Length, 100);
}

/**
  * @brief  Writes multiple data with I2C communication
  *         channel from MCU to TouchScreen.
  * @param  Addr: I2C address
  * @param  Reg: Register address
  * @param  Buffer: Pointer to data buffer
  * @param  Length: Length of the data
  * @retval None
  */
void SENSOR_IO_WriteMultiple(uint8_t Addr, uint8_t Reg, uint8_t *Buffer, uint16_t Length)
{
  I2Cx_WriteMultiple(&SENSOR_I2C_BUS_HANDLER, Addr, (uint16_t)Reg, I2C_MEMADD_SIZE_8BIT, Buffer, Length);
}


/*******************************************************************************
                            LINK OPERATIONS
*******************************************************************************/

/******************************** LINK NB  ********************************/
extern UART_HandleTypeDef NB_UART_BUS_HANDLER;

void NB_SendData(uint8_t *data, uint16_t len, uint8_t enter)
{
	uint8_t delimiter[2]={'\r', '\n'};
	while(NB_UART_BUS_HANDLER.gState != HAL_UART_STATE_READY);
	while(HAL_UART_Transmit_DMA(&NB_UART_BUS_HANDLER, data, len) == HAL_BUSY);
	while(NB_UART_BUS_HANDLER.gState != HAL_UART_STATE_READY);
	if (enter)
	{
		while(HAL_UART_Transmit_DMA(&NB_UART_BUS_HANDLER, delimiter, 1) == HAL_BUSY);
		while(NB_UART_BUS_HANDLER.gState != HAL_UART_STATE_READY);
	}
}

void HAL_UART_TxCpltCallback(UART_HandleTypeDef *huart)
{
	if ((huart->gState == HAL_UART_STATE_BUSY_TX) && (HAL_IS_BIT_SET(huart->Instance->CR3, USART_CR3_DMAT)))
	  {
		/* Disable TXEIE and TCIE interrupts */
		CLEAR_BIT(huart->Instance->CR1, (USART_CR1_TXEIE | USART_CR1_TCIE));
	    CLEAR_BIT(huart->Instance->CR3, USART_CR3_DMAT);
	    /* Abort the UART DMA Tx channel */
	    if(huart->hdmatx != NULL) HAL_DMA_Abort(huart->hdmatx);
	    /* At end of Tx process, restore huart->gState to Ready */
	    huart->gState = HAL_UART_STATE_READY;
	  }
}

void HAL_UART_RxCpltCallback(UART_HandleTypeDef *huart)
{
	if (huart->Instance == CMDLINE_UART) CmdLine_SaveByte(rxData);
	if (huart->Instance == NB_UART) NB_SaveByte(&rxNB);
	if (huart->Instance == GNSS_UART) NMEA_StoreByte(&rxGNSS);
}

void HAL_UART_ErrorCallback(UART_HandleTypeDef *huart)
{
	if (huart->ErrorCode != HAL_UART_ERROR_ORE) return;
	if (huart->Instance == CMDLINE_UART) CmdLine_SaveByte((uint8_t)(huart->Instance->RDR & (uint8_t)0x00FF));
	if (huart->Instance == NB_UART) NB_SaveByte((uint8_t*)(huart->Instance->RDR & (uint8_t)0x00FF));
	if (huart->Instance == GNSS_UART) NMEA_StoreByte((uint8_t*)(huart->Instance->RDR & (uint8_t)0x00FF));
}

int _write (int fd, char *ptr, int len)
{
	while(CMDLINE_UART_BUS_HANDLER.gState != HAL_UART_STATE_READY);
	HAL_UART_Transmit_DMA(&CMDLINE_UART_BUS_HANDLER, (uint8_t *)ptr, len);
	while(CMDLINE_UART_BUS_HANDLER.gState != HAL_UART_STATE_READY);
	return len;
}

void BOARD_LowPower(uint16_t on)
{
	if (on)
	{
		/* Board Power Down Required */
		BSP_LIGHTSENSE_LowPower(BSP_I2C_ADDRESS_LIGHTSENSE, 1);
		BSP_ACCELERO_LowPower(1);
	}
	else
	{
		/* Board Power Up Required */
		BSP_LIGHTSENSE_LowPower(BSP_I2C_ADDRESS_LIGHTSENSE, 0);
		BSP_ACCELERO_LowPower(0);
	}
}

/**
  * @brief EXTI line detection callbacks
  * @param GPIO_Pin: Specifies the pins connected EXTI line
  * @retval None
  */
void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin)
{
  if (GPIO_Pin == HALLSENSE.GPIO_Pin) HALLSENSE_ReadState();
}


