/**
  ******************************************************************************
  * @file           : button.c
  * @brief          : Button Driver
  * @author			: ELKYS servis s.r.o. (2019)
  ******************************************************************************
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of ELKYS servis s.r.o.  nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */
#include "main.h"
#include "board.h"
#include "configuration.h"
#include "button.h"
#include "packet.h"
#include "nmea.h"

//extern NAND_HandleTypeDef hnand1;
BUTTONInfo_TypeDef BUTTON[MAX_NUM_OF_BUTTONS];
#define BUTTON_STATE(b) HAL_GPIO_ReadPin(BUTTON[b].GPIO_Port, BUTTON[b].GPIO_Pin)

void BUTTON_Init(GPIO_TypeDef* GPIOx, uint16_t GPIO_Pin, uint8_t button)
{
	if (button >= MAX_NUM_OF_BUTTONS) return;
	BUTTON[button].GPIO_Port = GPIOx;
	BUTTON[button].GPIO_Pin = GPIO_Pin;
	BUTTON[button].Timer = 0;
	BUTTON[button].State = BUTTON_WAIT_TO_PRESS;
	BUTTON[button].Press = BUTTON_NONE;
}

BUTTONPress_TypeDef BUTTON_KeyPressed(uint8_t button)
{
	BUTTONPress_TypeDef tmp = BUTTON[button].Press;
	if  (BUTTON[button].Press != BUTTON_NONE) BUTTON[button].Press = BUTTON_NONE;
	return tmp;
}
void BUTTON_Handler(void)
{
	uint8_t i;
	int16_t acc[3] = {0, 0, 0};
	uint16_t light[2] = {0, 0};
	uint8_t val[12] = {0}, len;
	float tmp_f = 0;

	for (i=0; i<MAX_NUM_OF_BUTTONS; i++)
	{
		switch(BUTTON_KeyPressed(i))
		{
		case BUTTON_NONE:
			/* DO NOTHING */
			break;
		case BUTTON_LONGPRESS_DETECTED:
			LED_SetState(LED_GREEN, 0x0F);
			LED_SetState(LED_RED, 0x0F);
			printf("LONGPRESS DETECTED\n");
			break;
		case BUTTON_LONGPRESS:
			printf("LONGPRESS ACTIVATE\n");
			NMEA_ToggleON();
			break;
		case BUTTON_DOUBLE_CLICK:
			LED_SetState(LED_RED, 0x01);
			LED_SetState(LED_GREEN, 0x0F);
			printf("---( DOUBLECLICK-INFO )---------------------\n");
			PACKET_Start(); //create packet to send
			PACKET_AddElement(ELEMENT_ID_SIGNAL, ELEMENT_TYPE_1BYTE, &NBInfo.RxLevel, 1);

			/* Read Temperature Sensor */
			tmp_f = BSP_TSENSOR_ReadTemperature();
			printf("Temperature: %.2f\n", tmp_f);
			PACKET_AddElement(ELEMENT_ID_TEMPERATURE, ELEMENT_TYPE_4BYTE, (uint8_t *)&tmp_f, 4);

			/* Read Humidity Sensor */
			tmp_f = BSP_HSENSOR_ReadHumidity();
			printf("Humidity: %.2f\n", tmp_f);
			PACKET_AddElement(ELEMENT_ID_HUMIDITY, ELEMENT_TYPE_4BYTE, (uint8_t *)&tmp_f, 4);

			/* Read Accelerometer Sensor */
			BSP_ACCELERO_AccGetXYZ(acc);
			printf("ACC: x=%d y=%d z=%d\n", acc[0], acc[1], acc[2]);
			PACKET_AddElement(ELEMENT_ID_ACCX, ELEMENT_TYPE_2BYTE, (uint8_t *)&acc[0], 2);
			PACKET_AddElement(ELEMENT_ID_ACCY, ELEMENT_TYPE_2BYTE, (uint8_t *)&acc[1], 2);
			PACKET_AddElement(ELEMENT_ID_ACCZ, ELEMENT_TYPE_2BYTE, (uint8_t *)&acc[2], 2);

			/* Read Light Sensor */
			BSP_LIGHTSENSE_GetValue(BSP_I2C_ADDRESS_LIGHTSENSE, light);
			printf("LIGHT CPU: IR1+2=%d IR1=%d\n", light[0], light[1]);
			PACKET_AddElement(ELEMENT_ID_LIGHT12, ELEMENT_TYPE_2BYTE, (uint8_t *)&light[0], 2);
			PACKET_AddElement(ELEMENT_ID_LIGHT1, ELEMENT_TYPE_2BYTE, (uint8_t *)&light[1], 2);

			/* Send position to packet */
			printf("GNSS: %.6f %.6f\n", GNSSInfo.Latitude, GNSSInfo.Longitude);
			PACKET_AddElement(ELEMENT_ID_LATITUDE, ELEMENT_TYPE_4BYTE, (uint8_t *)&GNSSInfo.Latitude, 4);
			PACKET_AddElement(ELEMENT_ID_LONGITUDE, ELEMENT_TYPE_4BYTE, (uint8_t *)&GNSSInfo.Longitude, 4);

			PACKET_Finish();	/* Packet is finished and ready to save in buffer */
			break;
		case BUTTON_CLICK:
			LED_SetState(LED_GREEN, 0x01);
			PACKET_Start();
			len = sprintf((char *)&val[0],"CLICK");
			PACKET_AddElement(ELEMENT_ID_DIAGNOSTIC, ELEMENT_TYPE_STRING, val, len);
			PACKET_Finish();
			break;
		}
	}//for
}

void BUTTON_TimerHandler(void)
{
	uint8_t button = 0;
	for (button=0; button<MAX_NUM_OF_BUTTONS; button++)
	{
		if (BUTTON[button].Timer) BUTTON[button].Timer--;
		switch(BUTTON[button].State)
		{
			case BUTTON_WAIT_TO_PRESS:
				if (BUTTON_STATE(button) == BUTTON_IS_PRESSED)
				{
					BUTTON[button].State = BUTTON_WAIT_TO_LONGPRESS;
					BUTTON[button].Timer = BUTTON_LONGPRESS_IN_MS;
				}
				break;
			case BUTTON_WAIT_TO_LONGPRESS:
				if (BUTTON[button].Timer == 0)
				{
					/*LONG PRESS TIMER IS EXPIRED */
					BUTTON[button].State = BUTTON_WAIT_TO_RELEASE;
					BUTTON[button].Press = 	BUTTON_LONGPRESS_DETECTED;

				}
				if (BUTTON_STATE(button) != BUTTON_IS_PRESSED)
				{
					/* BUTTON IS RELEASED */
					BUTTON[button].State = BUTTON_WAIT_TO_CLICK;
					BUTTON[button].Timer = BUTTON_DOUBLECLICK_WAIT_IN_MS;
				}
				break;
			case BUTTON_WAIT_TO_CLICK:
				if (BUTTON[button].Timer == 0)
				{
					/* CLICK FINISHED*/
					BUTTON[button].State = BUTTON_WAIT_TO_PRESS;
					BUTTON[button].Press = 	BUTTON_CLICK;
	#if defined(BUTTON_DEBUG)
					printf("CLICK\n");
	#endif
				}

				if (BUTTON_STATE(button) == BUTTON_IS_PRESSED)
				{
					/* DOUBLE CLICK IS DETECTED */
					BUTTON[button].State = BUTTON_WAIT_TO_DOUBLECLICK;
				}
				break;
			case BUTTON_WAIT_TO_DOUBLECLICK:
				if (BUTTON_STATE(button) != BUTTON_IS_PRESSED)
				{
					/* DOUBLE CLICK FINISHED*/
					BUTTON[button].State = BUTTON_WAIT_TO_PRESS;
					BUTTON[button].Timer = 0;
					BUTTON[button].Press = 	BUTTON_DOUBLE_CLICK;
	#if defined(BUTTON_DEBUG)
					printf("DOUBLE CLICK\n");
	#endif
				}
				break;
			case BUTTON_WAIT_TO_RELEASE:
				if (BUTTON_STATE(button) != BUTTON_IS_PRESSED)
				{
					BUTTON[button].State = BUTTON_WAIT_TO_PRESS;
					BUTTON[button].Timer = 0;
					BUTTON[button].Press = 	BUTTON_LONGPRESS;
	#if defined(BUTTON_DEBUG)
					printf("LONG PRESS\n");
	#endif
				}
				break;
		}
	}//for
}
